(** This file contains a few utilities for working with sequence
    profiles. *)

(** Type of a profile to hold matrices and vectors *)
type profile = {
  len:int;				(** Length of the profile *)
  seq:string;				(** Sequence from the profile *)
  pssm:float array array;		(** Position specific scoring matrix *)
  psfm:float array array;		(** Position specific frequency matrix *)
  ipp:float array;			(** Information per position *)
  relwgt:float array;			(** Relative weight *)
};;

let empty_profile = {
  len=0; seq=""; pssm=[|[||]|]; psfm=[|[||]|]; ipp=[||]; relwgt=[||];
}

let prof_size = 20;;			(** Number of columns in a PSSM/PSFM *)
let ident x = x;;

open Scanf;;

(** Read a single profile from file. If [gkform] is true, then the
    profile is George's modified PSI-BLAST output *)
let read_profile ?(gkform=false) fname len = 
  let f = Scanning.from_file fname in
  let seq = String.create len in
  let pssm = Array.make_matrix len prof_size 0.0 in
  let psfm = Array.make_matrix len prof_size 0.0 in
  let ipp = Array.make len 0.0 in
  let relwgt = Array.make len 0.0 in

  bscanf f "%s@\n" ignore;		(*Skip first line*)
  if not gkform then bscanf f "%_s@\n%s@\n" ignore; (* Standard PSI-BLAST has two other ingorable line *)
  for i=0 to len-1 do
    seq.[i] <- bscanf f " %_d %c " ident;
    for j=0 to prof_size-1 do
      pssm.(i).(j) <- bscanf f " %f" ident;
    done;
    for j=0 to prof_size-1 do
      psfm.(i).(j) <- bscanf f " %f" ident;
    done;
    ipp.(i) <- bscanf f " %f" ident;
    relwgt.(i) <- bscanf f " %f" ident;
  done;
  (* Finish scanning the profile so that the file closes, a hack to
     appease the kludgy construction of ocaml's Scanbuf *)
  while not (Scanning.end_of_input f) do
    bscanf f " %s" ignore;
  done;
  {len=len; seq=seq; pssm=pssm; psfm=psfm; ipp=ipp; relwgt=relwgt};;

(** Take a slice of a profile *)
let profile_slice p start len =
  {len = len;
   seq = String.sub p.seq start len;
   pssm = Array.sub p.pssm start len;
   psfm = Array.sub p.psfm start len;
   ipp = Array.sub p.ipp start len;
   relwgt = Array.sub p.relwgt start len;
  };;

(** Take a slice of the profile starting with [start] and ending with [stop] *)
let profile_slice_idx p start stop =
  let len = stop-start+1 in
  profile_slice p start len;;
